function [f,fd,h,f_x,a,ssr,V] = fdll(Y,X,N,x,h,sa,kern,cv)
% Estimate Partially Linear Fixed-Effects Model using FD (First Differencing) 
%   Y_it = a_i + f(X_it) + e_it, i=1:N, t=1:T
% Input: 
%   Y: the vectorized panel data of dependent variable y_it
%       Y = [y_1; y_2; ...; y_N], where y_i corresponds to group i
%   X: the vectorized panel data of independent variable x_it
%       X = [x_1; x_2; ...; x_N], where x_i corresponds to group i
%   N: the number of groups
%   x: evaluation points of f()
%   h: window width
%   sa: choose sample augmentation (1) or not (0)
%   kern: choose kernel: 
%       0 (default): normal; 1: Epanechinikov; 2: Box-shaped; 3:
%       triangular; 4: cosine
%   cv: 0: no cv; 1: choose h by cross-validation.
%
% Output:
%   f: the value of f(x). If x is a vector, then f is also a vector.
%   a: the fixed effects (a_i)
%   ssr: sum of squared residual 
%
% Junhui Qian
% August, 2009
% jhqian@sjtu.edu.cn
%

n = length(Y);
T = n/N;
nn = N*(T-1);

if (nargin < 6 || isempty(sa)) %
    sa = 1;
end
if (nargin < 7 || isempty(kern)) %
    kern = 0;
end                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             
if (nargin < 8 || isempty(cv)) %
    cv = 0;
end

switch kern
    case 0
        kernel = @normal;
    case 1
        kernel = @epanechinikov;
    case 2
        kernel = @box;
    case 3
        kernel = @triangle;
    case 4
        kernel = @cosine;
end

y = reshape(diff(reshape(Y,T,N)),nn,1);
Z = reshape(X,T,N);
Z = [reshape(Z(2:T,:),nn, 1) reshape(Z(1:T-1,:),nn,1)]; % (X_it, X_i,t-1)
if sa
    y = [y;-y]; Z = [Z;[Z(:,2) Z(:,1)]]; % sample augmentation
    n1 = 2*nn;
else
    n1 = nn;
end

if (nargin < 5 || isempty(h)) %
    h = std(X)/n1^(1/5);
    if cv
%        options = optimset('TolX',max(1e-4,0.1/n1^(1/5)));
%        [h1,fval,exitflag] = fminbnd(@cvfun,0.1*h,10*h,options,Y,X,N,sa,kern);
%        [h1,fval,exitflag] = fminsearch(@cvfun,h,options,Y,X,N,sa,kern);
        [h1,fval,exitflag] = fmincon(@cvfuncp,h,[],[],[],[],h/10,10*h,[],[],Y,X,N,sa,kern);
        if (h1<0 || exitflag <= 0)
            disp('Minimization failed. Use default window width instead.')
        else
            h = h1;
        end
    end
end

k = length(x);
f = zeros(k,1);
fd = zeros(k,1);
for i = 1:k
    [m,m1] = fdcm(y,Z,x(i),X,h,kernel);
    f(i) = mean(m);
    fd(i) = mean(m1(:,1));
end
if (nargout > 3) %
    f_x = spline(x,f,X);
%    f_x = zeros(n,1);
%    for i = 1:n
%        m = fdcm(y,Z,X(i),X,h,kernel);
%        f_x(i) = mean(m);
%    end
end
if (nargout > 4) %
    e = Y - f_x;
    a = mean(reshape(e,T,N))';
end
if (nargout > 5) %
    ssr = (e-kron(a,ones(T,1)))'*(e-kron(a,ones(T,1))); 
end
if (nargout > 6) %
    V = zeros(k,1);
%    er = y - (spline(x,f,Z(:,1)) - spline(x,f,Z(:,2)));
    FX = reshape(diff(reshape(f_x,T,N)),nn,1);
    er = y(1:nn) - FX;
    for i = 1:k
        w = reshape(feval(kernel,(x(i)*ones(n*nn,1)-kron(Z(1:nn,1),ones(n,1)))/h)/h .* feval(kernel,(repmat(X,nn,1)-kron(Z(1:nn,2),ones(n,1)))/h)/h,n,nn); % n by nn
        p2 = mean(w,2); % p2(x(i),X^*), n by 1
        s = w./repmat(p2,1,nn); % n by nn
        th = mean(s)'; % nn by 1
        V(i) = sum(th.^2 .* er.^2)/((T-1)^2*N^(6/5));
    end
end

function y = cvfun(h,Y,X,N,sa,kern)
n = length(Y);
T = n/N;
index = kron([1:N]',ones(T,1));
y = 0;
for i = 1:N
    X0 = X(index ~= i); Y0 = Y(index ~= i);
    X1 = X(index == i); Y1 = Y(index == i);
    f = fdll(Y0,X0,N-1,X1,h,sa,kern);
    error = (Y1(2:T)-Y1(1:T-1)) - (f(2:T)-f(1:T-1));
    y = y + sum(error.^2);
end
y = log(y/(N*(T-1)));
if isnan(y) || isinf(y)
    y = 1e10;
end

function cp = cvfuncp(h,Y,X,N,sa,kern)
switch kern
    case 0
        kernel = @normal;
    case 1
        kernel = @epanechinikov;
    case 2
        kernel = @box;
    case 3
        kernel = @triangle;
    case 4
        kernel = @cosine;
end
n = length(Y);
T = n/N; nn = (T-1)*N;
fx = fdll(Y,X,N,X,h,sa);
FX = reshape(diff(reshape(fx,T,N)),nn,1);
y = reshape(diff(reshape(Y,T,N)),nn,1);
er = y - FX;
EPE = er'*er/nn;
e = Y - fx;
a = mean(reshape(e,T,N))';
s2 = (e-kron(a,ones(T,1)))'*(e-kron(a,ones(T,1)))/n; 
cp = EPE + 2*s2*feval(kernel,0)/(h*nn);

%%%%%%%%%%%%%%%%%%%% kernels %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function y = normal(z)
% Normal
y =  (1/sqrt(2*pi)) * exp(-0.5*z.^2);

function f = epanechinikov(z)
%EPANECHINIKOV Epanechinikov's asymptotically optimal kernel.
a = sqrt(5);
f = .75 * (1 - .2*z.^2) / a .* (abs(z)<=a);

function f = box(z)
%BOX    Box-shaped kernel
a = sqrt(3);
f = (abs(z)<=a) ./ (2 * a);

function f = cosine(z)
%TRIANGLE Triangular kernel.
%z = max(-1, min(z,1));
f = pi/4*cos(pi/2*z).*(abs(z)<=1);

function f = triangle(z)
%TRIANGLE Triangular kernel.
a = sqrt(6);
z = abs(z);
f = (z<=a) .* (1 - z/a) / a; 

