function [f,fd,h,effects,ssr,V,f_x] = fdll2(Y,X,N,x,h,sa,kern,cv)
% Estimate Partially Linear Two-Way Effects Panel Model using FD (First Differencing) 
%   Y_it = a_i + b_t + f(X_it) + e_it, i=1:N, t=1:T
% Input: 
%   Y: the vectorized panel data of dependent variable y_it
%       Y = [y_1; y_2; ...; y_N], where y_i corresponds to group i
%   X: the vectorized panel data of independent variable x_it
%       X = [x_1; x_2; ...; x_N], where x_i corresponds to group i
%   N: the number of groups
%   x: evaluation points of f()
%   h: window width
%   sa: choose sample augmentation (1) or not (0)
%   kern: choose kernel: 
%       0 (default): normal; 1: Epanechinikov; 2: Box-shaped; 3:
%       triangular; 4: cosine
%   cv: 0: no cv; 1: choose h by cross-validation.
%
% Output:
%   f: the value of f(x). If x is a vector, then f is also a vector.
%   a: the fixed effects (a_i)
%   b: the time fixed effects (b_t)
%   ssr: sum of squared residual 
%   V: asymptotic variance
%
% Junhui Qian
% August, 2009
% jhqian@sjtu.edu.cn
%

n = length(Y);
T = n/N;
nn = (N-1)*(T-1);
if (nargin < 8 | isempty(cv))
    cv = 0;
end
if (nargin < 6 | isempty(sa)) %
    sa = 1;
end
if sa
    n1 = 4*(N-1)*(T-1);
    adj = 4;
else
    n1 = (N-1)*(T-1);
    adj = 1;
end

if (nargin < 5 | isempty(h)) %
    h = std(X)/n1^(1/5); % optimal window width 
    if cv
%        options = optimset('TolX',max(1e-4,0.1/n1^(1/5)));
%        [h1,fval,exitflag] = fminbnd(@cvfun,0.1*h,10*h,options,Y,X,N,sa,kern);
%        [h1,fval,exitflag] = fminsearch(@cvfun,h,[],Y,X,N,sa,kern);
    [h1,fval,exitflag] = fmincon(@cvfuncp,h,[],[],[],[],h/5,5*h,[],[],Y,X,N,sa,kern);
        if (h1<0 | exitflag <= 0)
            fprintf('Minimization failed. Use default window width instead.')
        else
            h = h1;
        end
    end
end
if (nargin < 7 | isempty(kern)) %
    kern = @normal;
end

y = reshape(Y,T,N);
y11 = y(2:T,2:N); y10 = y(1:T-1,2:N); y01=y(2:T,1:N-1); y00 = y(1:T-1,1:N-1);
y = reshape((y11-y10) - (y01-y00), (T-1)*(N-1), 1);
Z = reshape(X,T,N);
Z1 = [reshape(Z(1:T-1,2:N),(T-1)*(N-1),1) reshape(Z(2:T,1:N-1),(T-1)*(N-1),1) reshape(Z(1:T-1,1:N-1),(T-1)*(N-1),1)];
Z = [reshape(Z(2:T,2:N),(T-1)*(N-1), 1) Z1]; % (X_it, X_i,t-1)
if sa
    y = [y;y;-y;-y]; Z = [Z;[Z(:,4) Z(:,3) Z(:,2) Z(:,1)];[Z(:,2) Z(:,1) Z(:,4) Z(:,3)];[Z(:,3) Z(:,4) Z(:,1) Z(:,2)]]; % sample augmentation
end


k = length(x);
f = zeros(k,1);
fd = zeros(k,1);
for i = 1:k
    [m,m1] = fdcm(y,Z,x(i),Z1,h,kern,h,kern);
    f(i) = mean(m);
    fd(i) = mean(m1(:,1));
end

if (nargout > 3) %
    f_x = zeros(n,1);
%    f_x = spline(x,f,X);
    for i = 1:n
        m = fdcm(y,Z,X(i),Z1,h,kern,h,kern);
        f_x(i) = mean(m);
    end
    b = cumsum(mean((diff(reshape(Y,T,N))-diff(reshape(f_x,T,N)))')');
    a = cumsum(mean((diff(reshape(Y,T,N)')-diff(reshape(f_x,T,N)'))')');
    e = Y - f_x - kron([0;a],ones(T,1)) - repmat([0;b],N,1);
    c = mean(e); 
    effects = [c;a;b];
    e = e-c;
end
if (nargout > 4) %
    ssr = e'*e; 
end
if (nargout > 5) %
    V = zeros(k,1);
    em = reshape(e,T,N);
    e11 = em(2:T,2:N); e10 = em(1:T-1,2:N); e01=em(2:T,1:N-1); e00 = em(1:T-1,1:N-1);
    er = reshape((e11-e10) - (e01-e00), (T-1)*(N-1), 1);
%    FX = reshape(f_x,T,N);
%    FX11 = FX(2:T,2:N); FX10 = FX(1:T-1,2:N); FX01=FX(2:T,1:N-1); FX00 = FX(1:T-1,1:N-1);
%    FX = reshape((FX11-FX10) - (FX01-FX00), (T-1)*(N-1), 1);
%    er = y(1:nn) - FX;
    for i = 1:k
        w = reshape(prod([feval(kern,(x(i)*ones(n1*nn,1)-kron(Z(1:nn,1),ones(n1,1)))/h)/h feval(kern,(repmat(Z(:,2:4),nn,1)-kron(Z(1:nn,2:4),ones(n1,1)))/h)/h^3]')',n1,nn);
        p = mean(w')'; %n1 by 1%
        s = w./repmat(p,1,nn);
        th = mean(s)';
        V(i) = sum(th.^2 .* er.^2)/((T-1)^2*(N-1)^(6/5));
    end
end

function y = cvfun(h,Y,X,N,sa,kern)

n = length(Y);
T = n/N;
index = kron([1:N]',ones(T,1));
y = 0;
for i = 1:N
    X0 = X(index ~= i); Y0 = Y(index ~= i);
    X1 = X(index == i); Y1 = Y(index == i);
    f = fdll2(Y0,X0,N-1,X1,h,sa,kern);
    error = (Y1(2:T)-Y1(1:T-1)) - (f(2:T)-f(1:T-1));
    y = y + sum(error.^2);
end
y = log(y/(N*(T-1)));
if isnan(y) | isinf(y)
    y = 1e10;
end

function cp = cvfuncp(h,Y,X,N,sa,kernel)
if (nargin < 6 | isempty(kernel)) %
    kernel = @normal;
end
n = length(Y);
T = n/N; nn = (T-1)*(N-1);
fx = fdll(Y,X,N,X,h,sa);
b = cumsum(mean((diff(reshape(Y,T,N))-diff(reshape(fx,T,N)))')');
a = cumsum(mean((diff(reshape(Y,T,N)')-diff(reshape(fx,T,N)'))')');
e = Y - fx - kron([0;a],ones(T,1)) - repmat([0;b],N,1);
c = mean(e); 
e = e-c;
s2 = e'*e/n;

em = reshape(e,T,N);
e11 = em(2:T,2:N); e10 = em(1:T-1,2:N); e01=em(2:T,1:N-1); e00 = em(1:T-1,1:N-1);
er = reshape((e11-e10) - (e01-e00), (T-1)*(N-1), 1);

EPE = er'*er/nn;

cp = EPE + 4*s2*feval(kernel,0)/(nn*h);

%%%%%%%%%%%%%%%%%%%% kernels %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function y = normal(z)
% Normal
y =  (1/sqrt(2*pi)) * exp(-0.5*z.^2);

function f = epanechinikov(z)
%EPANECHINIKOV Epanechinikov's asymptotically optimal kernel.
a = sqrt(5);
f = .75 * (1 - .2*z.^2) / a .* (abs(z)<=a);

function y = normal4(z)
y = (1/sqrt(2*pi)) * exp(-0.5*(z.*z)) .* ((1.5 - 0.5 * (z.*z)));


